@@ -1,6 +1,6 @@
 ##
 ## R package abclass developed by Wenjie Wang <wang@wwenjie.org>
-## Copyright (C) 2021-2022 Eli Lilly and Company
+## Copyright (C) 2021-2025 Eli Lilly and Company
 ##
 ## This file is part of the R package abclass.
 ##
@@ -33,6 +33,5 @@
 ##' @importFrom Rcpp sourceCpp
 ##' @useDynLib abclass
 ##'
-##' @docType package
-##' @name abclass-package
-NULL
+##' @keywords internal
+"_PACKAGE"
