@@ -1,6 +1,6 @@
 ##
 ## R package abclass developed by Wenjie Wang <wang@wwenjie.org>
-## Copyright (C) 2021-2022 Eli Lilly and Company
+## Copyright (C) 2021-2025 Eli Lilly and Company
 ##
 ## This file is part of the R package abclass.
 ##
@@ -17,21 +17,15 @@
 
 ##' @importFrom stats BIC
 ##' @export
-BIC.logistic_suplasso <- function(object, ...)
+BIC.supclass <- function(object, ...)
 {
     nll <- attr(object$coefficients, "negLogL")
+    if (is.null(nll)) {
+        return(NULL)
+    }
     k <- apply(object$coefficients, 3L, function(a) {
         sum(abs(a) > .Machine$double.eps)
     })
     nobs <- length(object$category$y)
     log(nobs) * k + 2 * nll * nobs
 }
-
-##' @export
-BIC.logistic_supscad <- BIC.logistic_suplasso
-
-##' @export
-BIC.supclass <- function(object, ...)
-{
-    NULL
-}
